@@ -2,7 +2,8 @@ import os
 import numpy as np
 import datetime
 import csv
-import xlrd
+import openpyxl
+from openpyxl.utils import get_column_letter
 import xlsxwriter
 from dataclasses import dataclass, asdict
 
@@ -176,7 +177,17 @@ class Cable:
     Tc0_Nb3Sn: np.ndarray = np.array([])
     Bc20_Nb3Sn: np.ndarray = np.array([])
 
-def CompareLEDETParameters(FileA, FileB):
+def read_row(workSheet, Nrow, St = False):
+    rowValues = np.array([])
+    row = workSheet[Nrow]
+    for cell in row:
+        if not St:
+            if isinstance(cell.value, str): continue
+        rowValues = np.append(rowValues, cell.value)
+    rowValues = rowValues[rowValues != None]
+    return rowValues
+
+def CompareLEDETParameters(FileA, FileB, Precision = 1E-5):
     Diff = 0
 
     P_a = ParametersLEDET()
@@ -193,9 +204,10 @@ def CompareLEDETParameters(FileA, FileB):
 
         P_a_a = P_a.getAttribute("Inputs", attribute)
         P_b_a = P_b.getAttribute("Inputs", attribute)
-        if isinstance(P_a_a, float):
+
+        if isinstance(P_a_a, float) or isinstance(P_a_a, int):
             if P_a_a != P_b_a:
-                if Block: print("Found difference in Parameter {}".format(attribute))
+                if Block: print("Found difference in Parameter {}, A: {}, B: {}".format(attribute,P_a_a, P_b_a))
                 Block = 0
                 Diff = 1
         elif len(P_a_a) != len(P_b_a):
@@ -204,59 +216,76 @@ def CompareLEDETParameters(FileA, FileB):
                 Block = 0
                 print('Parameter {} of A, {} has not the same length as Parameter of B, {}'.format(attribute, len(P_a_a), len(P_b_a)))
         else:
+            Pos = []
             for k in range(len(P_a_a)):
                 try:
                     if P_a_a[k] != P_b_a[k]:
-                        Diff = 1
-                        if Block:
-                            print("Found difference in Parameter {}".format(attribute))
-                            Block = 0
-                        print("Position: {}".format(k))
-                except:
-                    for j in range(P_a_a.shape[1]):
-                        if P_a_a[k,j] != P_b_a[k,j]:
+                        if abs(P_a_a[k]-P_b_a[k])>Precision:
                             Diff = 1
                             if Block:
                                 print("Found difference in Parameter {}".format(attribute))
                                 Block = 0
-                            if CC<5:
-                                print("Position: Row: {}, Col: {}".format(k, j))
-                                CC = CC +1
-                            elif CC==5:
-                                print("Many errors in this array. Continue to next.")
-                                CC = CC + 1
+                            Pos.append(k)
+                except:
+                    for j in range(P_a_a.shape[1]):
+                        if P_a_a[k,j] != P_b_a[k,j]:
+                            if abs(P_a_a[k,j] - P_b_a[k,j]) > Precision:
+                                Diff = 1
+                                if Block:
+                                    print("Found difference in Parameter {}".format(attribute))
+                                    Block = 0
+                                Pos.append([k,j])
+
+            if len(Pos)>0:
+                if len(Pos)<10:
+                    print("Different Positions: {}".format(Pos))
+                else:
+                    print("Many values are different (>10)")
+
+
     ## Check Options
     for attribute in P_a.Options.__annotations__:
         Block = 1
         P_a_a = P_a.getAttribute("Options", attribute)
         P_b_a = P_b.getAttribute("Options", attribute)
-        if isinstance(P_a_a, float):
+        if isinstance(P_a_a, float) or isinstance(P_a_a, int):
             if P_a_a != P_b_a:
-                Diff = 1
-                if Block: print("Found difference in Parameter {}".format(attribute))
+                if Block: print("Found difference in Parameter {}, A: {}, B: {}".format(attribute, P_a_a, P_b_a))
                 Block = 0
+                Diff = 1
         elif len(P_a_a) != len(P_b_a):
             Diff = 1
             if Block:
                 Block = 0
-                print('Parameter {} of A, {} has not the same length as Parameter of B, {}'.format(attribute, len(P_a_a), len(P_b_a)))
+                print(
+                    'Parameter {} of A, {} has not the same length as Parameter of B, {}'.format(attribute, len(P_a_a),
+                                                                                                 len(P_b_a)))
         else:
+            Pos = []
             for k in range(len(P_a_a)):
                 try:
                     if P_a_a[k] != P_b_a[k]:
-                        if Block:
-                            print("Found difference in Parameter {}".format(attribute))
-                            Block = 0
-                        print("Position: {}".format(k))
-                        Diff = 1
-                except:
-                    for j in range(P_a_a.shape[1]):
-                        if P_a_a[k,j] != P_b_a[k,j]:
+                        if abs(P_a_a[k] - P_b_a[k]) > Precision:
+                            Diff = 1
                             if Block:
                                 print("Found difference in Parameter {}".format(attribute))
                                 Block = 0
-                            print("Position: Row: {}, Col: {}".format(k, j))
-                            Diff = 1
+                            Pos.append(k)
+                except:
+                    for j in range(P_a_a.shape[1]):
+                        if P_a_a[k, j] != P_b_a[k, j]:
+                            if abs(P_a_a[k, j] - P_b_a[k, j]) > Precision:
+                                Diff = 1
+                                if Block:
+                                    print("Found difference in Parameter {}".format(attribute))
+                                    Block = 0
+                                Pos.append([k, j])
+
+            if len(Pos) > 0:
+                if len(Pos) < 10:
+                    print("Different Positions: {}".format(Pos))
+                else:
+                    print("Many values are different (>10)")
 
     if Diff==0:
         print("Files are equal.")
@@ -307,18 +336,18 @@ class ParametersLEDET:
 
     def readLEDETExcel(self, file):
         ##File must be whole eos string
-        workbookVariables = xlrd.open_workbook(file)
+        workbookVariables = openpyxl.load_workbook(file, data_only=True)
 
         #Inputs
-        worksheetInputs = workbookVariables.sheet_by_name('Inputs')
-        lastAttribute = worksheetInputs.cell(0, 1).value
-        for i in range(worksheetInputs.nrows):
-            self.variablesInputs[str(worksheetInputs.cell(i, 1).value)] = str(worksheetInputs.cell(i, 0).value)
-            attribute = worksheetInputs.cell(i, 1).value
+        worksheetInputs = workbookVariables['Inputs']
+        lastAttribute = worksheetInputs.cell(1, 2).value
+        for i in range(1, worksheetInputs.max_row+1):
+            self.variablesInputs[str(worksheetInputs.cell(i, 2).value)] = str(worksheetInputs.cell(i, 1).value)
+            attribute = worksheetInputs.cell(i, 2).value
             try:
-                if (attribute == ''):
-                    if (type(worksheetInputs.cell(i, 2).value) != str):
-                        values = worksheetInputs.row_values(i, 2)
+                if (attribute == None):
+                    if worksheetInputs.cell(i, 3).value is not None:
+                        values = read_row(worksheetInputs, i)
                         values = np.array([k for k in values if(str(k))])
                         current = self.getAttribute(self.Inputs, lastAttribute)
                         current = np.vstack((current, values))
@@ -327,44 +356,44 @@ class ParametersLEDET:
                         continue
                 elif (type(self.getAttribute(self.Inputs, attribute)) == np.ndarray):
                     lastAttribute = attribute
-                    values = worksheetInputs.row_values(i, 2)
+                    values = read_row(worksheetInputs, i)
                     values = np.array([k for k in values if(str(k))])
                     self.setAttribute(self.Inputs, attribute, values)
                 else:
-                    value = worksheetInputs.cell(i, 2).value
+                    value = worksheetInputs.cell(i, 3).value
                     self.setAttribute(self.Inputs, attribute, value)
             except TypeError as e:
                 if attribute in self.Inputs.__annotations__: raise e
                 print("Error with attribute: {}, continuing.".format(attribute))
         #Options
-        worksheetOptions = workbookVariables.sheet_by_name('Options')
-        for i in range(worksheetOptions.nrows):
-            self.variablesOptions[str(worksheetOptions.cell(i, 1).value)] = str(worksheetOptions.cell(i, 0).value)
-            attribute = worksheetOptions.cell(i, 1).value
+        worksheetOptions = workbookVariables['Options']
+        for i in range(1, worksheetOptions.max_row+1):
+            self.variablesOptions[str(worksheetOptions.cell(i, 2).value)] = str(worksheetOptions.cell(i, 1).value)
+            attribute = worksheetOptions.cell(i, 2).value
             try:
                 if (type(self.getAttribute(self.Options, attribute)) == np.ndarray):
-                    values = worksheetOptions.row_values(i, 2)
+                    values = read_row(worksheetOptions, i)
                     values = np.array([k for k in values if(str(k))])
                     self.setAttribute(self.Options, attribute, values)
                 else:
-                    value = worksheetOptions.cell(i, 2).value
+                    value = worksheetOptions.cell(i, 3).value
                     self.setAttribute(self.Options, attribute, value)
             except  TypeError as e:
                 if attribute in self.Options.__annotations__: raise e
                 print("Error with attribute: {}, continuing.".format(attribute))
-                #Plots
-        worksheetPlots = workbookVariables.sheet_by_name('Plots')
-        for i in range(worksheetPlots.nrows):
-            self.variablesPlots[str(worksheetPlots.cell(i, 1).value)] = str(worksheetPlots.cell(i, 0).value)
-            attribute = worksheetPlots.cell(i, 1).value
+        #Plots
+        worksheetPlots = workbookVariables['Plots']
+        for i in range(1, worksheetPlots.max_row+1):
+            self.variablesPlots[str(worksheetPlots.cell(i, 2).value)] = str(worksheetPlots.cell(i, 1).value)
+            attribute = worksheetPlots.cell(i, 2).value
             try:
                 if (type(self.getAttribute(self.Plots, attribute)) == np.ndarray):
-                    values = worksheetPlots.row_values(i, 2)
+                    values = read_row(worksheetPlots, i, St=True)[2:]
                     values = np.array([k for k in values if(str(k))])
                     self.setAttribute(self.Plots, attribute, values)
                 else:
                     try:
-                        value = worksheetPlots.cell(i, 2).value
+                        value = worksheetPlots.cell(i, 3).value
                     except:
                         value = ''
                     self.setAttribute(self.Plots, attribute, value)
@@ -372,22 +401,24 @@ class ParametersLEDET:
                 print("Error with attribute: {}, continuing.".format(attribute))
         # Variables
         try:
-            worksheetVariables = workbookVariables.sheet_by_name('Variables')
-            for i in range(worksheetVariables.nrows):
-                self.variablesVariables[str(worksheetVariables.cell(i, 1).value)] = str(worksheetVariables.cell(i, 0).value)
-                attribute = worksheetVariables.cell(i, 1).value
+            worksheetVariables = workbookVariables['Variables']
+            for i in range(1, worksheetVariables.max_row+1):
+                self.variablesVariables[str(worksheetVariables.cell(i, 2).value)] = str(worksheetVariables.cell(i, 1).value)
+                attribute = worksheetVariables.cell(i, 2).value
                 try:
                     if (type(self.getAttribute(self.Variables, attribute)) == np.ndarray):
-                        values = worksheetVariables.row_values(i, 2)
+                        if attribute != 'typeVariableToSaveTxt':  values = read_row(worksheetVariables, i, St = True)[2:]
+                        else:  values = read_row(worksheetVariables, i)
                         values = np.array([k for k in values if(str(k))])
                         self.setAttribute(self.Variables, attribute, values)
                     else:
-                        value = worksheetVariables.cell(i, 2).value
+                        value = worksheetVariables.cell(i, 3).value
                         self.setAttribute(self.Variables, attribute, value)
                 except TypeError as e:
                     if attribute in self.Variables.__annotations__: raise e
                     print("Error with attribute: {}, continuing.".format(attribute))
-        except xlrd.biffh.XLRDError as e:
+        except:
+            pass
             print("Error while reading Variables. Please check!")
 
     def __init__(self):
@@ -437,31 +468,31 @@ class ParametersLEDET:
         # Read variable names and descriptions
         fullfileName = ResourceReader.getResourcePath(fileDefaultParameters)
         # print(fullfileName) # for debug
-        workbookVariables = xlrd.open_workbook(fullfileName)
+        workbookVariables = openpyxl.load_workbook(fullfileName)
 
         # Load "Inputs" sheet
-        worksheetInputs = workbookVariables.sheet_by_name('Inputs')
+        worksheetInputs = workbookVariables['Inputs']
         variablesInputs = {}
-        for i in range(worksheetInputs.nrows):
-            variablesInputs[str(worksheetInputs.cell(i, 1).value)] = str(worksheetInputs.cell(i, 0).value)
+        for i in range(1, worksheetInputs.max_row+1):
+            variablesInputs[str(worksheetInputs.cell(i, 2).value)] = str(worksheetInputs.cell(i, 1).value)
 
         # Load "Options" sheet
-        worksheetOptions = workbookVariables.sheet_by_name('Options')
+        worksheetOptions = workbookVariables['Options']
         variablesOptions = {}
-        for i in range(worksheetOptions.nrows):
-            variablesOptions[str(worksheetOptions.cell(i, 1).value)] = str(worksheetOptions.cell(i, 0).value)
+        for i in range(1, worksheetOptions.max_row+1):
+            variablesOptions[str(worksheetOptions.cell(i, 2).value)] = str(worksheetOptions.cell(i, 1).value)
 
         # Load "Plots" sheet
-        worksheetPlots = workbookVariables.sheet_by_name('Plots')
+        worksheetPlots = workbookVariables['Plots']
         variablesPlots = {}
-        for i in range(worksheetPlots.nrows):
-            variablesPlots[str(worksheetPlots.cell(i, 1).value)] = str(worksheetPlots.cell(i, 0).value)
+        for i in range(1, worksheetPlots.max_row+1):
+            variablesPlots[str(worksheetPlots.cell(i, 2).value)] = str(worksheetPlots.cell(i, 1).value)
 
         # Load "Variables" sheet
-        worksheetVariables = workbookVariables.sheet_by_name('Variables')
+        worksheetVariables = workbookVariables['Variables']
         variablesVariables = {}
-        for i in range(worksheetVariables.nrows):
-            variablesVariables[str(worksheetVariables.cell(i, 1).value)] = str(worksheetVariables.cell(i, 0).value)
+        for i in range(1, worksheetVariables.max_row+1):
+            variablesVariables[str(worksheetVariables.cell(i, 2).value)] = str(worksheetVariables.cell(i, 1).value)
 
         # Set parameters
         self.setParameters(variablesInputs, variablesOptions, variablesPlots, variablesVariables)
@@ -957,16 +988,12 @@ class ParametersLEDET:
             else:
                 print("Preliminary consistency check was successful! - ", nameFileLEDET)
 
-        workbook = xlsxwriter.Workbook(nameFileLEDET)
-
-        if verbose:
-            print('### Write "Inputs" sheet ###')
-        self.writeLEDETInputsNew(workbook, "Inputs", self.Inputs, self.variablesInputs, verbose)
+        workbook = openpyxl.Workbook()
 
         if verbose:
             print('')
-            print('### Write "Options" sheet ###')
-        self.writeLEDETInputsNew(workbook, "Options", self.Options, self.variablesOptions, verbose)
+            print('### Write "Variables" sheet ###')
+        self.writeLEDETInputsNew(workbook, "Variables", self.Variables, self.variablesVariables, verbose)
 
         if verbose:
             print('')
@@ -975,11 +1002,17 @@ class ParametersLEDET:
 
         if verbose:
             print('')
-            print('### Write "Variables" sheet ###')
-        self.writeLEDETInputsNew(workbook, "Variables", self.Variables, self.variablesVariables, verbose)
+            print('### Write "Options" sheet ###')
+        self.writeLEDETInputsNew(workbook, "Options", self.Options, self.variablesOptions, verbose)
+
+        if verbose:
+            print('### Write "Inputs" sheet ###')
+        self.writeLEDETInputsNew(workbook, "Inputs", self.Inputs, self.variablesInputs, verbose)
 
         # Save the workbook
-        workbook.close()
+        std = workbook['Sheet']
+        workbook.remove(std)
+        workbook.save(nameFileLEDET)
 
         # Display time stamp and end run
         currentDT = datetime.datetime.now()
@@ -1007,45 +1040,50 @@ class ParametersLEDET:
             :type verbose: bool
             :return:
         """
-        worksheet = book.add_worksheet(sheet)
-        cell_format = book.add_format({'bold': False, 'font_name': 'Calibri', 'font_size': 11})
+        book.create_sheet(index = 1 , title = sheet)
+        for s in range(len(book.sheetnames)):
+            if book.sheetnames[s] == sheet: break
+        book.active = s
+        sheet1 = book.active
 
         # Write to the sheet of the workbook
         currentRow = 0
         self.updateBlankrows(variableGroup)
         for attribute in variableGroup.__annotations__:
-            if (attribute == "BlankRows"):
-                continue
+            if (attribute == "BlankRows"): continue
             if (attribute == "overwrite_f_internalVoids_inGroup"):
                 ofiVg = self.getAttribute(variableGroup, attribute)
-                if len(ofiVg) == 0:
-                    #currentRow = currentRow + 1
-                    continue
+                if len(ofiVg) == 0: continue
             if (attribute == "overwrite_f_externalVoids_inGroup"):
                 ofiVg = self.getAttribute(variableGroup, attribute)
-                if len(ofiVg) == 0:
-                    #currentRow = currentRow + 1
-                    continue
+                if len(ofiVg) == 0: continue
             if currentRow in variableGroup.BlankRows[1]:
                 currentRow = currentRow + 1
+                sheet1.append([None])
+
             varDesc = variableLabels.get(str(attribute))
-            worksheet.write(currentRow, 0, varDesc, cell_format)
-            worksheet.write(currentRow, 1, attribute, cell_format)
             varType = type(self.getAttribute(variableGroup, attribute))
             if varType == np.ndarray:
                 if self.getAttribute(variableGroup, attribute).ndim > 1:
                     for i in range(self.getAttribute(variableGroup, attribute).shape[1]):
-                        worksheet.write_row(currentRow, 2, self.getAttribute(variableGroup, attribute)[i, :], cell_format)
+                        values = self.getAttribute(variableGroup, attribute)[i].tolist()
+                        if i == 0 : sheet1.append([varDesc, attribute]+ values)
+                        else : sheet1.append([None, None]+ values)
                         currentRow = currentRow + 1
-                else:
-                    worksheet.write_row(currentRow, 2, self.getAttribute(variableGroup, attribute), cell_format)
-                    currentRow = currentRow + 1
-            elif varType == list:
-                worksheet.write_row(currentRow, 2, self.getAttribute(variableGroup, attribute), cell_format)
-                currentRow = currentRow + 1
-            else:
-                worksheet.write(currentRow, 2, self.getAttribute(variableGroup, attribute), cell_format)
-                currentRow = currentRow + 1
-            worksheet.set_column(0, 0, 80)
-            worksheet.set_column(1, 1, 40)
-            worksheet.set_column(2, 1000, 20)
\ No newline at end of file
+                    continue
+
+            currentRow = currentRow + 1
+            if isinstance(self.getAttribute(variableGroup, attribute), np.ndarray):
+                values = np.array(self.getAttribute(variableGroup, attribute)).tolist()
+            elif isinstance(self.getAttribute(variableGroup, attribute), list):
+                values = self.getAttribute(variableGroup, attribute)
+            else: values = [self.getAttribute(variableGroup, attribute)]
+            sheet1.append([varDesc, attribute]+ values)
+
+            width = [80.7109375, 40.7109375, 20.7109375]
+            sheet1.column_dimensions['A'].width = width[0]
+            sheet1.column_dimensions['B'].width = width[1]
+            maxc = get_column_letter(sheet1.max_column)
+            for i in range(3,sheet1.max_column+1):
+                cl = get_column_letter(i)
+                sheet1.column_dimensions[cl].width = width[2]
\ No newline at end of file
