@@ -115,6 +115,21 @@ class CircuitFrequencyModelling:
         self.F_SCALE_L = F_SCALE_L
         self.F_SCALE_R = F_SCALE_R
 
+        flag_R_Par = a.cell(ind, circs_columns.index('flag_R_Par')).value
+        R_Par = a.cell(ind, circs_columns.index('R_Par (Ohm)')).value
+
+        self.flag_R_Par = flag_R_Par
+        self.R_Par = R_Par
+
+        # Eddy current loops
+        flag_E_loops = a.cell(ind, circs_columns.index('flag_E_loops')).value
+        E_loops = np.array(a.cell(ind, circs_columns.index('E_loops [L (H), R (Ohm), k]')).value.split(','))
+        E_loops = E_loops.astype(np.float)
+        E_loops = E_loops.reshape(np.int_(len(E_loops) / 3), 3)
+
+        self.flag_E_loops = flag_E_loops
+        self.E_loops = E_loops
+
         self.customPrint('Number of magnet types:')
         self.customPrint(len(MAG_TYPE))
 
@@ -156,6 +171,11 @@ class CircuitFrequencyModelling:
         F_SCALE_L = self.F_SCALE_L
         F_SCALE_R = self.F_SCALE_R
 
+        flag_R_Par = self.flag_R_Par
+        R_Par = self.R_Par
+        flag_E_loops = self.flag_E_loops
+        E_loops = self.E_loops
+
         netlist = Netlist("")
 
         libraryPaths = []
@@ -168,6 +188,9 @@ class CircuitFrequencyModelling:
         global_param = []
         global_values = []
 
+        E_param_param = []
+        E_param_values = []
+
         # PARAM for all non magnet components
 
         # Array components
@@ -185,9 +208,28 @@ class CircuitFrequencyModelling:
         global_values = global_values + [str.format('{}', R_SER)]
 
         # Scales
-        global_param = global_param + ['F_SCALE_C_GROUND_circuit', 'F_SCALE_C_T2T_circuit', 'F_SCALE_L_circuit', 'F_SCALE_R_circuit']
-        global_values = global_values + [str.format('{}', F_SCALE_C_GROUND), str.format('{}', F_SCALE_C_T2T), str.format('{}', F_SCALE_L), str.format('{}', F_SCALE_R)]
+        if model == "magnet":
+            global_param = global_param + ['F_SCALE_C_GROUND', 'F_SCALE_L', 'F_SCALE_R']
+            global_values = global_values + [str.format('{}', F_SCALE_C_GROUND), str.format('{}', F_SCALE_L), str.format('{}', F_SCALE_R)]
+        else:
+            global_param = global_param + ['F_SCALE_C_GROUND', 'F_SCALE_C_T2T', 'F_SCALE_L', 'F_SCALE_R']
+            global_values = global_values + [str.format('{}', F_SCALE_C_GROUND), str.format('{}', F_SCALE_C_T2T), str.format('{}', F_SCALE_L), str.format('{}', F_SCALE_R)]
 
+        if flag_E_loops:
+            self.customPrint("WARNING: Eddy current loops added, will cause errors if less eddy current loops are present in the used library files.")
+            for i in range(len(E_loops)):
+                E_param_param = E_param_param + [str.format('L_EDDY{}', i + 1), str.format('R_EDDY{}', i + 1),
+                                                 str.format('K_EDDY{}', i + 1)]
+                E_param_values = E_param_values + [str.format('{}', E_loops[i][0]), str.format('{}', E_loops[i][1]),
+                                                   str.format('{}', E_loops[i][2])]
+
+            global_param = global_param + E_param_param
+            global_values = global_values + E_param_values
+
+        if flag_R_Par:
+            self.customPrint("WARNING: Parallel resistance added, will cause errors if the parallel resistance isn't present in the used library files.")
+            global_param = global_param + ['R_Par']
+            global_values = global_values + [str.format('{}', R_Par)]
 
         global_param = a.create_string_array(gateway, global_param)
         global_values = a.create_string_array(gateway, global_values)
@@ -198,7 +240,7 @@ class CircuitFrequencyModelling:
         s = "* " + str(MAG_TYPE)
         netlist.add(CommentElement(s))
 
-        netlist.add(CommentElement("* .STEP PARAM R_Par 10k, 10k, 100k"))
+        netlist.add(CommentElement("* .STEP PARAM R_Par 10k, 100k, 10k"))
 
         #placeholder nodes
         nodes = gateway.new_array(gateway.jvm.String, 2)
@@ -233,8 +275,26 @@ class CircuitFrequencyModelling:
                 nodes[0], nodes[1] = str.format("{}mid" , k) , str.format("{}out" , k)
                 nodesX[0], nodesX[1], nodesX[2] = nodes[0], nodes[1], GROUND_NODE
                 attribute = MAG_TYPE[i]+"_"+model
-                magnet_parameters = a.create_string_array(gateway, ["F_SCALE_C_GROUND","F_SCALE_C_T2T","F_SCALE_L","F_SCALE_R"])
-                magnet_values = a.create_string_array(gateway, ["F_SCALE_C_GROUND_circuit","F_SCALE_C_T2T_circuit","F_SCALE_L_circuit","F_SCALE_R_circuit"])
+                if model == "magnet":
+                    magnet_param = ["F_SCALE_C_GROUND","\n+ F_SCALE_L","F_SCALE_R"] #"\n+" is added to make new lines when needed
+                    magnet_val = ["F_SCALE_C_GROUND","F_SCALE_L","F_SCALE_R"]
+                else:
+                    magnet_param = ["F_SCALE_C_GROUND","F_SCALE_C_T2T","\n+ F_SCALE_L","F_SCALE_R"] #"\n+" is added to make new lines when needed
+                    magnet_val = ["F_SCALE_C_GROUND","F_SCALE_C_T2T","F_SCALE_L","F_SCALE_R"]
+
+                if flag_E_loops:
+                    for E_i in range(len(E_loops)):
+                        magnet_param = magnet_param + [str.format("\n+ L_EDDY{}", E_i + 1), str.format("R_EDDY{}", E_i + 1),
+                                                         str.format("K_EDDY{}", E_i + 1)]
+                        magnet_val = magnet_val + [str.format("L_EDDY{}", E_i + 1), str.format("R_EDDY{}", E_i + 1),
+                                                         str.format("K_EDDY{}", E_i + 1)]
+
+                if flag_R_Par:
+                    magnet_param = magnet_param + ["\n+ R_Par"]
+                    magnet_val = magnet_val + ["R_Par"]
+
+                magnet_parameters = a.create_string_array(gateway, magnet_param)
+                magnet_values = a.create_string_array(gateway,magnet_val)
                 netlist.add(ParameterizedElement(name, nodesX, attribute, magnet_parameters, magnet_values))
                 #R_PAR
                 name=str.format("R_PAR_{}_{}_N_{}", i + 1, MAG_TYPE[i], j + 1)
@@ -283,6 +343,9 @@ class CircuitFrequencyModelling:
         s = model + '-based netlist file generated.'
         print(s)
 
+        #remove empty "+" lines
+        netlistAsListString = [sub.replace('\n+ \n+', '\n+') for sub in netlistAsListString]
+
         #change probe line
         probe_ind = netlistAsListString.index('\n.PROBE')
         netlistAsListString[probe_ind] = '\n* Configuration file\n.INC configurationFileFrequency.cir'
@@ -712,7 +775,7 @@ class FrequencyModelling:
         # This function uses the 'calculateInductance' function from the 'SelfMutualInductanceCalculation' dataclass
         # to calculate the mutual-inductance (M) matrixes based on turns and groups.
         # The calculated turn based M-matrix is compared to the one from the LEDET file, to check if they are a match.
-        # The amount of devation is printed. However since deviation aren't necesarilly cause for concern the code
+        # The amount of deviation is printed. However since deviation aren't necesarilly cause for concern the code
         # continous, no matter the result of the deviation test.
         # Some corrections are made, so that the length of the magnet, as well as the yoke effect is taken into account.
         # Finally the coupling coefficients (k) matrixes are calculated since PSpice needs those.
@@ -783,10 +846,10 @@ class FrequencyModelling:
         # Deviation test
         M_dev = abs(M_InductanceBlock_m - LEDET.Inputs.M_InductanceBlock_m)  # deviation of each element
         M_dev_rel = M_dev / abs(M_InductanceBlock_m)  # relative deviation of each element
-        nDev = sum(sum(M_dev_rel > dev_max))  # number of devations in the entire matrix
+        nDev = sum(sum(M_dev_rel > dev_max))  # number of deviations in the entire matrix
         sizeM = len(M_dev) * len(M_dev[0])
         self.customPrint('')
-        self.customPrint('Number of devations: ' + str(nDev) + ' out of ' + str(sizeM))
+        self.customPrint('Number of deviations: ' + str(nDev) + ' out of ' + str(sizeM))
 
         # Iron yoke effect
         M_groups_m = M_groups0_m * LEDET.Inputs.fL_L[0]
@@ -802,7 +865,8 @@ class FrequencyModelling:
         self.Total_Inductance = L_mag0
         self.customPrint('')
         self.customPrint('Total inductance:')
-        self.customPrint(L_mag0)
+        s = str(L_mag0)+' H'
+        self.customPrint(s)
 
         # Changed to M to L for easier comparison to older code
         L_turns = M_turns
@@ -853,6 +917,10 @@ class FrequencyModelling:
                  raise Exception('k>1 for non single-strand magnet')
                  sys.exit(1)
 
+        if len(k_turns) >= 1414: #A length of above 1414 will give a matrix of around 2 million values, which will require about 1 million lines
+            self.customPrint('')
+            self.customPrint('Warning: Netlist will be over 1 million lines long, might cause memory shortage errors')
+
         return L_turns, L_groups, k_turns, k_groups
 
     def Resistance(self):
@@ -1100,7 +1168,7 @@ class FrequencyModelling:
         C_turns, ind_turns, dc = self.CapSum(C_half, ind_turns0, np.zeros((len(ind_turns0),1))) # the To input/output is 0 in all cases, since they are grounded
         C_groups, ind_groups, dc = self.CapSum(C_half, ind_groups0, np.zeros((len(ind_groups0),1)))
 
-        self.Total_CapacitanceToGround = sum(C_turns)
+        self.Total_Capacitance = sum(C_turns)
 
 
         return C_turns, ind_turns, C_groups, ind_groups
@@ -1338,13 +1406,13 @@ class FrequencyModelling:
         netlist.add(CommentElement("**** Global parameters ****"))
         netlist.add(GlobalParameterElement(globalParameters_Parameters, globalParameters_Values))
 
-        netlist.add(CommentElement("* .STEP PARAM R_Par 10k, 10k, 100k"))
+        netlist.add(CommentElement("* .STEP PARAM R_Par 10k, 100k, 10k"))
 
         nodesL = gateway.new_array(gateway.jvm.String, len(L), 2)
         nodesR = gateway.new_array(gateway.jvm.String, len(R), 2)
 
 
-        GROUND_NODE = "0"
+        GROUND_NODE = "00_mGND"
 
         for i in range(len(L)):
             if i == 0:
@@ -1396,7 +1464,11 @@ class FrequencyModelling:
         # Add capacitance to ground
         netlist.add(CommentElement(" Capacitance to ground"))
         for i in range(len(C)):
-            nodesC[i][0] = str.format("{}out", reverseOrder[C_ind[i]-1]);
+            nodeC = reverseOrder[C_ind[i]-1]-1 # Capacitance at the input of the cell, aka output of previous cell
+            if nodeC == 0:
+                nodesC[i][0] = str.format("{}in", nodeC + 1) #First cell exception
+            else:
+                nodesC[i][0] = str.format("{}out", nodeC);
             nodesC[i][1] = GROUND_NODE;
             namesC[i] = str.format("C_{}", C_ind[i])
             nodes[0], nodes[1] = nodesC[i][1], nodesC[i][0]
@@ -1404,8 +1476,16 @@ class FrequencyModelling:
             netlist.add(GeneralElement(namesC[i], nodes, value))
 
         for i in range(len(Capacity_T2T_new)):
-            nodesT2TC[i][0] = str.format("{}out", reverseOrder[Contact_From_new[i]-1]);
-            nodesT2TC[i][1] = str.format("{}out", reverseOrder[Contact_To_new[i]-1]);
+            nodeFrom = reverseOrder[Contact_From_new[i]-1]-1 # Capacitance at the input of the cell, aka output of previous cell
+            nodeTo = reverseOrder[Contact_To_new[i]-1]-1 # Capacitance at the input of the cell, aka output of previous cell
+            if nodeFrom == 0:
+                nodesT2TC[i][0] = str.format("{}in", nodeFrom + 1) #First cell exception
+            else:
+                nodesT2TC[i][0] = str.format("{}out", nodeFrom);
+            if nodeTo == 0:
+                nodesT2TC[i][1] = str.format("{}in", nodeTo + 1) #First cell exception
+            else:
+                nodesT2TC[i][1] = str.format("{}out", nodeTo);
 
         namesT2TC = gateway.new_array(gateway.jvm.String, len(Capacity_T2T_new))
 
@@ -1478,17 +1558,24 @@ class FrequencyModelling:
         value = "AC 1"
         netlist.add(GeneralElement(name, nodes, value));
 
+
+        netlist.add(CommentElement(" Connection between local ground and last node"));
+        name = "V_localGround"
+        nodes[0], nodes[1] = "00_mGND", str.format("{}out",len(L))
+        value = "0"
+        netlist.add(GeneralElement(name, nodes, value));
+
         # Voltage measurement
         netlist.add(CommentElement(" Voltage measurement"));
         name = "E_total_voltage"
-        nodes[0], nodes[1] = GROUND_NODE, "0total_voltage"
+        nodes[0], nodes[1] = "0", "0total_voltage"
         value = "VALUE = {V(" + str.format("{}out" , len(L)) + ")-V(1in)}"
         netlist.add(GeneralElement(name, nodes, value));
 
-        # Add a resistor to ground for a bias point calculation
+        # Add a resistor between local and global ground for a bias point calculation
         netlist.add(CommentElement(" Resistance to ground"));
         name = "R_GND"
-        nodes[0], nodes[1] = nodesR[len(L) - 1][1], GROUND_NODE
+        nodes[0], nodes[1] = "0", GROUND_NODE
         value = "1e6"
         netlist.add(GeneralElement(name, nodes, value));
 
@@ -1541,15 +1628,18 @@ class FrequencyModelling:
             subckt = '.subckt ' + subname + ' 00_IN 00_OUT 00_mGND\n'
             Lib = list([subckt])
             Lib.extend(netlistAsListString[start+1:end])
-            Lib.extend(['\n.ends'])
-            paramInd = netlistAsListString.index('**** Global parameters ****')
-            firstCellInd = netlistAsListString.index('* Cell 1')
-            Lib = Lib[:paramInd-1] + Lib[firstCellInd-1:] # Remove everyting related to parameters (WHY -1?!)
-            #stepInd = [i for i, s in enumerate(Lib) if '.STEP PARAM' in s]
-            #for i in range(len(stepInd)): #doesn't run if the stepInd is empty
-            #    Lib.pop(stepInd[len(stepInd)-1-i]) #go through from back to front, so that the indexes still correspond to the correct string
-            Lib = [sub.replace('(0', '(00_mGND') for sub in Lib]
-            Lib = [sub.replace(' 0)', ' 00_mGND)') for sub in Lib]
+            Lib.extend(['\n.ends '])
+            #paramInd = netlistAsListString.index('**** Global parameters ****')
+            #firstCellInd = netlistAsListString.index('* Cell 1')
+            #Lib = Lib[:paramInd-1] + Lib[firstCellInd-1:] # Remove everyting related to parameters (WHY -1?!)
+            stepInd = [i for i, s in enumerate(Lib) if '.STEP PARAM' in s]
+            for i in range(len(stepInd)): #doesn't run if the stepInd is empty
+                Lib.pop(stepInd[len(stepInd)-1-i]) #go through from back to front, so that the indexes still correspond to the correct string
+
+            Lib = [sub.replace('.PARAM', '+ PARAMS:') for sub in Lib]
+
+            #Lib = [sub.replace('(0', '(00_mGND') for sub in Lib]
+            #Lib = [sub.replace(' 0)', ' 00_mGND)') for sub in Lib]
             Lib = [sub.replace('1in', '00_IN') for sub in Lib]
             Lib = [sub.replace(last, '00_OUT') for sub in Lib]
 
@@ -1564,3 +1654,213 @@ class FrequencyModelling:
             print('Netlist generation done, file generation disabled.')
 
         return netlist, netlistAsListString
+
+    def netlistMagnet(self):
+        # Creates a simple netlist only consisting of the total inductance, total resistance and total capacitance to ground.
+        flag_writeFile = self.flag_writeFile
+        flag_R_Par = self.flag_R_Par
+        R_Par = self.R_Par
+        flag_E_loops = self.flag_E_loops
+        E_loops = self.E_loops
+        F_SCALE_C_GROUND = self.F_SCALE_C_GROUND
+        F_SCALE_L = self.F_SCALE_L
+        F_SCALE_R = self.F_SCALE_R
+        pointsPerDecade = self.pointsPerDecade
+        startFrequency = self.startFrequency
+        stopFrequency = self.stopFrequency
+        nameMagnet = self.nameMagnet
+
+        path_gateway = self.eospath + 'SWAN_projects/steam-notebooks/steam/*'
+
+        # Launch a Gateway in a new Java process, this returns port
+        port = launch_gateway(classpath=path_gateway)
+        # JavaGateway instance is connected to a Gateway instance on the Java side
+        gateway = JavaGateway(gateway_parameters=GatewayParameters(port=port))
+        # Get STEAM API Java classes
+        MutualInductance = gateway.jvm.component.MutualInductance
+        Netlist = gateway.jvm.netlist.Netlist
+        CommentElement = gateway.jvm.netlist.elements.CommentElement
+        GeneralElement = gateway.jvm.netlist.elements.GeneralElement
+        GlobalParameterElement = gateway.jvm.netlist.elements.GlobalParameterElement
+        ACSolverElement = gateway.jvm.netlist.solvers.ACSolverElement
+        CircuitalPreconditionerSubcircuit = gateway.jvm.preconditioner.CircuitalPreconditionerSubcircuit
+        TextFile = gateway.jvm.utils.TextFile
+        CSVReader = gateway.jvm.utils.CSVReader
+
+        netlist = Netlist("")
+
+        globalParameters_Parameters = ['F_SCALE_C_GROUND', 'F_SCALE_L', 'F_SCALE_R']
+        globalParameters_Values = [str.format('{}', F_SCALE_C_GROUND), str.format('{}', F_SCALE_L), str.format('{}', F_SCALE_R)]
+
+        E_param_param = []
+        E_param_values = []
+
+        if flag_E_loops:
+            for i in range(len(E_loops)):
+                E_param_param = E_param_param + [str.format('L_EDDY{}', i + 1), str.format('R_EDDY{}', i + 1),
+                                                 str.format('K_EDDY{}', i + 1)]
+                E_param_values = E_param_values + [str.format('{}', E_loops[i][0]), str.format('{}', E_loops[i][1]),
+                                                   str.format('{}', E_loops[i][2])]
+
+            globalParameters_Parameters = globalParameters_Parameters + E_param_param
+            globalParameters_Values = globalParameters_Values + E_param_values
+
+        if flag_R_Par:
+            globalParameters_Parameters = globalParameters_Parameters + ['R_Par']
+            globalParameters_Values = globalParameters_Values + [str.format('{}', R_Par)]
+
+        globalParameters_Parameters = a.create_string_array(gateway, globalParameters_Parameters)
+        globalParameters_Values = a.create_string_array(gateway, globalParameters_Values)
+
+        netlist.add(CommentElement("**** Global parameters ****"))
+        netlist.add(GlobalParameterElement(globalParameters_Parameters, globalParameters_Values))
+
+        netlist.add(CommentElement("* .STEP PARAM R_Par 10k, 100k, 10k"))
+
+        GROUND_NODE = "1out"
+
+        nodes = gateway.new_array(gateway.jvm.String, 2)
+
+        # Add inductor
+        nameL = "L_total"
+        nodes[0], nodes[1] = "1in" , "1mid"
+        value = '{' + str(self.Total_Inductance) + '*F_SCALE_L}'
+        netlist.add(GeneralElement(nameL, nodes, value))
+
+        # Add resistance in series with the inductor
+        nameR = "R_total"
+        nodes[0], nodes[1] = "1mid" , "1out"
+        value = '{' + str(self.Total_Resistance) + '*F_SCALE_R}'
+        netlist.add(GeneralElement(nameR, nodes, value))
+
+        # Add capacitance to ground from input
+        nameC = "C_total"
+        nodes[0], nodes[1] = "1in", GROUND_NODE
+        value = '{' + str(self.Total_Capacitance) + '*F_SCALE_C_GROUND*0.40528473456}' # added 1/(pi/2)^2 correction
+        netlist.add(GeneralElement(nameC, nodes, value))
+
+
+        # Add parallel resistance
+        if flag_R_Par:
+            netlist.add(CommentElement("* Parallel resistance"));
+            name = "R_Par";
+            nodes[0], nodes[1] = "1in", "1out"
+            value = '{R_Par}';
+            netlist.add(GeneralElement(name, nodes, value));
+
+        # Add eddy current loops
+        if flag_E_loops:
+            nodesR_E = gateway.new_array(gateway.jvm.String, 2)
+            nodesL_E = gateway.new_array(gateway.jvm.String, 2)
+            nodesG_E = gateway.new_array(gateway.jvm.String, 2)
+
+            netlist.add(CommentElement(" Eddy current loops"));
+            for i in range(len(E_loops)):
+                netlist.add(CommentElement(str.format("* Loop {}", i + 1)));
+
+                nameR = str.format('R_EDDY{}', i + 1)
+                nameL = str.format('L_EDDY{}', i + 1)
+                nameG = str.format('R_EDDY{}_GND', i + 1)
+
+                nodesR_E[0], nodesR_E[1] = str.format('{}E_L', i + 1), str.format('{}E_R', i + 1)
+                nodesL_E[0], nodesL_E[1] = str.format('{}E_R', i + 1), str.format('{}E_L', i + 1)
+                nodesG_E[0], nodesG_E[1] = str.format('{}E_L', i + 1), '0'
+
+                valueL = '{' + nameL + '}'
+                valueR = '{' + nameR + '}'
+                valueK = '{' + str.format('K_EDDY{}', i + 1) + '}'
+                valueG = '1E12'
+
+                netlist.add(GeneralElement(nameR, nodesR_E, valueR));
+                netlist.add(GeneralElement(nameL, nodesL_E, valueL));
+                netlist.add(GeneralElement(nameG, nodesG_E, valueG));
+
+                netlist.add(CommentElement(' Coupling coefficients'));
+                name = str.format("K_E_{}", i + 1)
+                nodes[0], nodes[1] = nameL, "L_total"
+                value = valueK
+                netlist.add(GeneralElement(name, nodes, value))
+
+        # Voltage source
+        netlist.add(CommentElement(" Voltage source"));
+        name = "V_AC"
+        nodes[0], nodes[1] = "1out", "1in"
+        value = "AC 1"
+        netlist.add(GeneralElement(name, nodes, value));
+
+        # Voltage measurement
+        netlist.add(CommentElement(" Voltage measurement"));
+        name = "E_total_voltage"
+        nodes[0], nodes[1] = "0", "0total_voltage"
+        value = "VALUE = {V(1out)-V(1in)}"
+        netlist.add(GeneralElement(name, nodes, value));
+
+        # Add a resistor to ground for a bias point calculation
+        netlist.add(CommentElement(" Resistance to ground"));
+        name = "R_GND"
+        nodes[0], nodes[1] = "1out", "0"
+        value = "1e6"
+        netlist.add(GeneralElement(name, nodes, value));
+
+        netlist.setSolver(ACSolverElement.Builder()
+                            .pointsPerDecade(pointsPerDecade)
+                            .startFrequency(startFrequency)
+                            .stopFrequency(stopFrequency)
+                            .build())
+
+        netlistAsListString = netlist.generateNetlistFile("BINARY")
+
+        if flag_writeFile:
+            cwd = os.getcwd()
+            Path(cwd + '/' + nameMagnet).mkdir(parents=True, exist_ok=True)
+            #cwd = cwd + '/' + nameMagnet
+            Circ = nameMagnet + '/' + nameMagnet + '_magnet.cir'
+            # subckt
+            subname = nameMagnet + '_magnet'
+
+            #change probe line
+            probe_ind = netlistAsListString.index('\n.PROBE')
+            netlistAsListString[probe_ind] = '\n* Configuration file\n.INC configurationFileFrequency.cir'
+            netlistAsListString = ListConverter().convert(netlistAsListString, gateway._gateway_client)
+
+            TextFile.writeMultiLine(Circ, netlistAsListString, False)
+
+            # Display time stamp
+            currentDT = datetime.datetime.now()
+            print(' ')
+            print('Time stamp: ' + str(currentDT))
+
+            print('Magnet-based netlist file generated.')
+
+            # subckt generation
+            start = netlistAsListString.index('* PSPICE Netlist Simulation File')
+            end = netlistAsListString.index('* Voltage source')
+            subckt = '.subckt ' + subname + ' 00_IN 00_OUT 00_mGND\n'
+            Lib = list([subckt])
+            Lib.extend(netlistAsListString[start+1:end])
+            Lib.extend(['\n.ends '])
+            #paramInd = netlistAsListString.index('**** Global parameters ****')
+            #firstCellInd = netlistAsListString.index('* Cell 1')
+            #Lib = Lib[:paramInd-1] + Lib[firstCellInd-1:] # Remove everyting related to parameters (WHY -1?!)
+            stepInd = [i for i, s in enumerate(Lib) if '.STEP PARAM' in s]
+            for i in range(len(stepInd)): #doesn't run if the stepInd is empty
+                Lib.pop(stepInd[len(stepInd)-1-i]) #go through from back to front, so that the indexes still correspond to the correct string
+
+            Lib = [sub.replace('.PARAM', '+ PARAMS:') for sub in Lib]
+
+            #Lib = [sub.replace('(0', '(00_mGND') for sub in Lib]
+            #Lib = [sub.replace(' 0)', ' 00_mGND)') for sub in Lib]
+            Lib = [sub.replace('1in', '00_IN') for sub in Lib]
+            Lib = [sub.replace('1out', '00_OUT') for sub in Lib]
+
+            psub = 'subckt/Items/'
+            Path(cwd + '/' + psub).mkdir(parents=True, exist_ok=True)
+            #cwd = cwd + '/' + nameMagnet
+            Libname = psub + subname + '.lib'
+            java_Lib = ListConverter().convert(Lib, gateway._gateway_client)
+            TextFile.writeMultiLine(Libname, java_Lib, False)
+
+        else:
+            print('Netlist generation done, file generation disabled.')
+
+        return netlist, netlistAsListString
